@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2004 the xine project
+ * Copyright (C) 2000-2006 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -15,7 +15,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * Credits go 
  * - for the SPDIF A/52 sync part
@@ -24,9 +24,6 @@
  * for initial ALSA 0.9.x support.
  *     adding MONO/STEREO/4CHANNEL/5CHANNEL/5.1CHANNEL analogue support.
  * (c) 2001 James Courtier-Dutton <James@superbug.demon.co.uk>
- *
- * 
- * $Id: audio_alsa_out.c,v 1.153 2004/12/27 16:55:39 hadess Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -41,17 +38,13 @@
 #include <stdlib.h>
 #include <fcntl.h>
 #include <math.h>
+#ifdef HAVE_ALLOCA_H
 #include <alloca.h>
+#endif
 
 #define ALSA_PCM_NEW_HW_PARAMS_API
 #define ALSA_PCM_NEW_SW_PARAMS_API
-#ifdef HAVE_ALSA_ASOUNDLIB_H
-#include <alsa/asoundlib.h>
-#elif HAVE_SYS_ASOUNDLIB_H
-#include <sys/asoundlib.h>
-#else
-#error "required asoundlib.h neither in sys/ nor alsa/ - unable to compile."
-#endif
+#include <asoundlib.h>
 
 #include <sys/ioctl.h>
 #include <inttypes.h>
@@ -168,7 +161,6 @@ static int my_snd_mixer_wait(snd_mixer_t *mixer, int timeout) {
 static void *ao_alsa_handle_event_thread(void *data) {
   alsa_driver_t  *this = (alsa_driver_t *) data;
 
-  this->mixer.running = 1;
   do {
 
     if(my_snd_mixer_wait(this->mixer.handle, 333) > 0) {
@@ -223,6 +215,7 @@ static void *ao_alsa_handle_event_thread(void *data) {
 	xine_event_t              event;
 	xine_audio_level_data_t   data;
 	xine_stream_t            *stream;
+	xine_list_iterator_t      ite;
 	
 	this->mixer.right_vol = right_vol;
 	this->mixer.left_vol  = left_vol;
@@ -242,8 +235,9 @@ static void *ao_alsa_handle_event_thread(void *data) {
 	event.data_length = sizeof(data);
 	
 	pthread_mutex_lock(&this->class->xine->streams_lock);
-	for(stream = xine_list_first_content(this->class->xine->streams); 
-	    stream; stream = xine_list_next_content(this->class->xine->streams)) {
+	for(ite = xine_list_front(this->class->xine->streams); 
+	    ite; ite = xine_list_next(this->class->xine->streams, ite)) {
+	  stream = xine_list_get_value(this->class->xine->streams, ite);
 	  event.stream = stream;
 	  xine_event_send(stream, &event);
 	}
@@ -270,40 +264,18 @@ static long ao_alsa_get_volume_from_percent(int val, long min, long max) {
  * Error callback, we need to control this, 
  * error message should be printed only in DEBUG mode.
  */
-static void error_callback(const char *file, int line, 
+static void XINE_FORMAT_PRINTF(5, 6)
+	    error_callback(const char *file, int line, 
 			   const char *function, int err, const char *fmt, ...) {
 #ifdef DEBUG
   va_list   args;
-  char     *buf;
-  int       n, size = 100;
-  
-  printf("%s:%s:%d entered\n", __FILE__, __FUNCTION__, __LINE__ );
-  if(!(buf = xine_xmalloc(size))) 
-    return;
-  
-  while(1) {
-    
-    va_start(args, fmt);
-    n = vsnprintf(buf, size, fmt, args);
-    va_end(args);
-    
-    if(n > -1 && n < size)
-      break;
-    
-    if(n > -1)
-      size = n + 1;
-    else
-      size *= 2;
-    
-    if((buf = realloc(buf, size)) == NULL)
-      return;
-  }
-  
-   
-  printf("%s: %s() %s.\n", file, function, buf);
+  char     *buf = NULL;
 
-  if(buf)
-    free(buf);
+  va_start(args, fmt);
+  vasprintf(&buf, fmt, args);
+  va_end(args);
+  printf("%s: %s() %s.\n", file, function, buf);
+  free(buf);
 #endif
 }
 
@@ -332,6 +304,8 @@ static int ao_alsa_open(ao_driver_t *this_gen, uint32_t bits, uint32_t rate, int
   int                   err, dir;
   int                 open_mode=1; /* NONBLOCK */
   /* int                   open_mode=0;  BLOCK */
+  struct timeval start_time;
+  struct timeval end_time;
 
   snd_pcm_hw_params_alloca(&params);
   snd_pcm_sw_params_alloca(&swparams);
@@ -384,8 +358,21 @@ static int ao_alsa_open(ao_driver_t *this_gen, uint32_t bits, uint32_t rate, int
   this->bytes_in_buffer        = 0;
   /*
    * open audio device
+   * When switching to surround, dmix blocks the device some time, so we just keep trying for 0.8sec.
    */
-  err=snd_pcm_open(&this->audio_fd, pcm_device, direction, open_mode);      
+  gettimeofday(&start_time, NULL);
+  do {
+    err = snd_pcm_open(&this->audio_fd, pcm_device, direction, open_mode);      
+    gettimeofday(&end_time, NULL);
+    if( err == -EBUSY ) {
+      if( (double)end_time.tv_sec + 1E-6*end_time.tv_usec 
+          - (double)start_time.tv_sec - 1E-6*start_time.tv_usec > 0.8)
+        break;
+      else
+        usleep(10000);
+    }
+  } while( err == -EBUSY );
+  
   if(err <0 ) {                                                           
     xprintf (this->class->xine, XINE_VERBOSITY_LOG, 
 	     _("audio_alsa_out: snd_pcm_open() of %s failed: %s\n"), pcm_device, snd_strerror(err));               
@@ -471,6 +458,10 @@ static int ao_alsa_open(ao_driver_t *this_gen, uint32_t bits, uint32_t rate, int
 	     this->num_channels, err, snd_strerror(err));
     goto close;
   }
+#if SND_LIB_VERSION >= 0x010009
+  /* Restrict a configuration space to contain only real hardware rates */
+  err = snd_pcm_hw_params_set_rate_resample(this->audio_fd, params, 0);
+#endif
   /* set the stream rate [Hz] */
   dir=0;
   err = snd_pcm_hw_params_set_rate_near(this->audio_fd, params, &rate, &dir);
@@ -668,15 +659,19 @@ static int ao_alsa_delay (ao_driver_t *this_gen)  {
   struct timeval now;
   printf("audio_alsa_out:delay:ENTERED\n");
 #endif
-  err=snd_pcm_delay( this->audio_fd, &delay );
+  err = snd_pcm_delay( this->audio_fd, &delay );
+
 #ifdef LOG_DEBUG
   printf("audio_alsa_out:delay:delay all=%ld err=%d\n",delay, err);
   gettimeofday(&now, 0);
   printf("audio_alsa_out:delay: Time = %ld.%ld\n", now.tv_sec, now.tv_usec);
   printf("audio_alsa_out:delay:FINISHED\n");
 #endif
-  return delay;
 
+  if (err || (delay < 0))
+    delay = 0;
+
+  return delay;
 }
 
 #if 0
@@ -714,6 +709,19 @@ static void xrun(alsa_driver_t *this)
 }
 #endif
 
+/*
+ * resume from suspend
+ */
+static int resume(snd_pcm_t *pcm)
+{
+  int res;
+  while ((res = snd_pcm_resume(pcm)) == -EAGAIN)
+    sleep(1);
+  if (! res)
+    return 0;
+  return snd_pcm_prepare(pcm);
+}
+
 /*
  * Write audio data to output buffer (blocking using snd_pcm_wait)
  */
@@ -738,6 +746,15 @@ static int ao_alsa_write(ao_driver_t *this_gen, int16_t *data, uint32_t count) {
 #endif
   snd_pcm_status_alloca(&pcm_stat);
   state = snd_pcm_state(this->audio_fd);
+  if (state == SND_PCM_STATE_SUSPENDED) {
+    res = resume(this->audio_fd);
+    if (res < 0)
+      return 0;
+    state = snd_pcm_state(this->audio_fd);
+  } else if (state == SND_PCM_STATE_DISCONNECTED) {
+    /* the device is gone. audio_out.c handles it if we return something < 0 */
+    return -1;
+  }
   if (state == SND_PCM_STATE_XRUN) {
 #ifdef LOG_DEBUG
     printf("audio_alsa_out:write:XRUN before\n");
@@ -769,11 +786,11 @@ static int ao_alsa_write(ao_driver_t *this_gen, int16_t *data, uint32_t count) {
 #endif
       snd_pcm_status(this->audio_fd, pcm_stat);
       if ( snd_pcm_status_get_avail(pcm_stat) < number_of_frames) {
-        wait_result = snd_pcm_wait(this->audio_fd, 1000000);
+        wait_result = snd_pcm_wait(this->audio_fd, 1000);
 #ifdef LOG_DEBUG
         printf("audio_alsa_out:write:loop:wait_result=%d\n",wait_result);
 #endif
-        if (wait_result < 0) return 0;
+        if (wait_result <= 0) return 0;
       }
     }
     if (this->mmap != 0) {
@@ -787,7 +804,16 @@ static int ao_alsa_write(ao_driver_t *this_gen, int16_t *data, uint32_t count) {
       printf("audio_alsa_out:write:result=%ld:%s\n",result, snd_strerror(result));
 #endif
       state = snd_pcm_state(this->audio_fd);
-      if ( (state != SND_PCM_STATE_PREPARED) &&
+      if (state == SND_PCM_STATE_SUSPENDED) {
+	res = resume(this->audio_fd);
+	if (res < 0)
+	  return 0;
+	continue;
+      }
+      if (state == SND_PCM_STATE_DISCONNECTED) {
+        /* the device is gone. audio_out.c handles it if we return something < 0 */
+        return -1;
+      } else if ( (state != SND_PCM_STATE_PREPARED) &&
            (state != SND_PCM_STATE_RUNNING) &&
            (state != SND_PCM_STATE_DRAINING) ) {
         xprintf(this->class->xine, XINE_VERBOSITY_DEBUG, 
@@ -830,7 +856,11 @@ static int ao_alsa_write(ao_driver_t *this_gen, int16_t *data, uint32_t count) {
 static void ao_alsa_close(ao_driver_t *this_gen) {
   alsa_driver_t *this = (alsa_driver_t *) this_gen;
 
-  if(this->audio_fd) snd_pcm_close(this->audio_fd);
+  if(this->audio_fd) {
+    snd_pcm_nonblock(this->audio_fd, 0);
+    snd_pcm_drain(this->audio_fd);
+    snd_pcm_close(this->audio_fd);
+  }
   this->audio_fd = NULL;
   this->has_pause_resume = 0; /* This is set at open time */
 }
@@ -1271,6 +1301,8 @@ static void ao_alsa_mixer_init(ao_driver_t *this_gen) {
   if (send_events && found) {
     pthread_attr_t       pth_attrs;
     struct sched_param   pth_params;
+  
+    this->mixer.running = 1;
 
     pthread_attr_init(&pth_attrs);
     
@@ -1300,7 +1332,7 @@ static ao_driver_t *open_plugin (audio_driver_class_t *class_gen, const void *da
   char                *pcm_device;
   snd_pcm_hw_params_t *params;
   /* for usability reasons, keep this in sync with audio_oss_out.c */
-  static char         *speaker_arrangement[] = {"Mono 1.0", "Stereo 2.0", "Headphones 2.0", "Stereo 2.1",
+  static const char   *speaker_arrangement[] = {"Mono 1.0", "Stereo 2.0", "Headphones 2.0", "Stereo 2.1",
     "Surround 3.0", "Surround 4.0", "Surround 4.1", "Surround 5.0", "Surround 5.1", "Surround 6.0",
     "Surround 6.1", "Surround 7.1", "Pass Through", NULL};
   #define MONO		0
@@ -1319,6 +1351,8 @@ static ao_driver_t *open_plugin (audio_driver_class_t *class_gen, const void *da
   int speakers;
 
   this = (alsa_driver_t *) xine_xmalloc (sizeof (alsa_driver_t));
+  if (!this)
+    return NULL;
 
   this->class = class;
 
@@ -1403,6 +1437,7 @@ static ao_driver_t *open_plugin (audio_driver_class_t *class_gen, const void *da
           _("snd_pcm_open() failed:%d:%s\n"), err, snd_strerror(err)); 
     xine_log (this->class->xine, XINE_LOG_MSG,
           _(">>> Check if another program already uses PCM <<<\n")); 
+    free(this);
     return NULL; 
   }
 
@@ -1413,6 +1448,8 @@ static ao_driver_t *open_plugin (audio_driver_class_t *class_gen, const void *da
   if (err < 0) {
     xprintf (this->class->xine, XINE_VERBOSITY_DEBUG, 
 	     "audio_alsa_out: broken configuration for this PCM: no configurations available\n");
+    snd_pcm_close(this->audio_fd);
+    free(this);
     return NULL;
   }
   err = snd_pcm_hw_params_set_access(this->audio_fd, params,
@@ -1420,6 +1457,8 @@ static ao_driver_t *open_plugin (audio_driver_class_t *class_gen, const void *da
   if (err < 0) {
     xprintf (this->class->xine, XINE_VERBOSITY_DEBUG, 
 	     "audio_alsa_out: access type not available");
+    snd_pcm_close(this->audio_fd);
+    free(this);
     return NULL;
   }
 
@@ -1476,6 +1515,13 @@ static ao_driver_t *open_plugin (audio_driver_class_t *class_gen, const void *da
     this->capabilities |= AO_CAP_FLOAT32;
     xprintf(class->xine, XINE_VERBOSITY_LOG, _("32bit "));
   }
+  if (0 == (this->capabilities & (AO_CAP_FLOAT32 | AO_CAP_24BITS | AO_CAP_16BITS | AO_CAP_8BITS))) {
+    xprintf (this->class->xine, XINE_VERBOSITY_DEBUG,
+        "\naudio_alsa_out: no supported PCM format found\n");
+    snd_pcm_close(this->audio_fd);
+    free(this);
+    return NULL;
+  }
   if (!(snd_pcm_hw_params_test_channels(this->audio_fd, params, 1))) {
     this->capabilities |= AO_CAP_MODE_MONO;
     xprintf(class->xine, XINE_VERBOSITY_LOG, _("mono "));
@@ -1553,6 +1599,12 @@ static ao_driver_t *open_plugin (audio_driver_class_t *class_gen, const void *da
                                                "information on alsa devices."),
                                              10, NULL,
                                              NULL);
+  if (!this->mixer.name) {
+    if (this->audio_fd)
+      snd_pcm_close (this->audio_fd);
+    free(this);
+    return NULL;
+  }
 
   pthread_mutex_init(&this->mixer.mutex, NULL);
   ao_alsa_mixer_init(&this->ao_driver);
@@ -1631,6 +1683,8 @@ static void *init_class (xine_t *xine, void *data) {
   alsa_class_t        *this;
 
   this = (alsa_class_t *) xine_xmalloc (sizeof (alsa_class_t));
+  if (!this)
+    return NULL;
 
   this->driver_class.open_plugin     = open_plugin;
   this->driver_class.get_identifier  = get_identifier;
@@ -1650,7 +1704,7 @@ static ao_info_t ao_info_alsa = {
  * exported plugin catalog entry
  */
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */  
   { PLUGIN_AUDIO_OUT, AO_OUT_ALSA_IFACE_VERSION, "alsa", XINE_VERSION_CODE, &ao_info_alsa, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
