@@ -15,12 +15,10 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  */
 
 /*
- * $Id: demux_avi.c,v 1.229 2007/01/19 00:26:40 dgp85 Exp $
- *
  * demultiplexer for avi streams
  *
  * part of the code is taken from
@@ -46,7 +44,6 @@
  * expect to find the next A/V frame.  We periodically check if we can
  * read data from the file at that offset.  If we can, we append index
  * data for as many frames as we can read at the time.
- *
  */
 
 /*
@@ -55,7 +52,6 @@
  * Transcode's and xine's avi code comes from the same source and
  * still has a very similar architecture, so it wasn't much effort to
  * port it from transcode to xine.
- *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -194,8 +190,8 @@ typedef struct{
   uint32_t  audio_posb;      /* Audio position: byte within chunk */
 
 
+  int       wavex_len;
   xine_waveformatex *wavex;
-  int    wavex_len;
 
   audio_index_t  audio_idx;
 
@@ -251,28 +247,27 @@ typedef struct demux_avi_s {
   input_plugin_t      *input;
   int                  status;
 
-  avi_t               *avi;
-
-  int                  no_audio;
-
   uint32_t             video_step;
   uint32_t             AVI_errno;
 
+  /* seeking args backup */
+  int                  seek_start_time;
+  off_t                seek_start_pos;
+
+  avi_t               *avi;
+
   idx_grow_t           idx_grow;
 
-  int                  streaming;
-  int                  last_index_entry_type;
-  int                  has_index;
-  
-  /* seeking args backup */
-  int                  seek_request;
-  off_t                seek_start_pos;
-  int                  seek_start_time;
+  uint8_t              no_audio:1;
+
+  uint8_t              streaming:1;
+  uint8_t              has_index:1;
+
+  uint8_t              seek_request:1;
 
   /* discontinuity detection (only at seek) */
-  int                  send_newpts;
-  int                  buf_flag_seek;
-  
+  uint8_t              buf_flag_seek:1;
+  uint8_t              send_newpts:1;
 } demux_avi_t ;
 
 typedef struct {
@@ -334,13 +329,13 @@ typedef struct {
 /* bit 31 denotes a keyframe */
 static uint32_t odml_len (unsigned char *str)
 {
-   return LE_32(str) & 0x7fffffff;
+   return _X_LE_32(str) & 0x7fffffff;
 }
 
 /* if bit 31 is 0, its a keyframe */
 static uint32_t odml_key (unsigned char *str)
 {
-   return (LE_32(str) & 0x80000000)?0:0x10;
+   return (_X_LE_32(str) & 0x80000000)?0:0x10;
 }
 
 static void check_newpts (demux_avi_t *this, int64_t pts, int video) {
@@ -555,7 +550,7 @@ static int idx_grow(demux_avi_t *this, int (*stopper)(demux_avi_t *, void *),
       continue;
     }
 
-    chunk_len = LE_32(data + 4);
+    chunk_len = _X_LE_32(data + 4);
     this->idx_grow.nexttagoffset += PAD_EVEN(chunk_len + AVI_HEADER_SIZE);
 
 
@@ -778,7 +773,7 @@ static avi_t *AVI_init(demux_avi_t *this) {
       break; /* We assume it's EOF */
     }
 
-    n = LE_32(data + 4);
+    n = _X_LE_32(data + 4);
     n = PAD_EVEN(n);
     next_chunk = this->idx_grow.nexttagoffset + 8 + n;
     
@@ -851,7 +846,7 @@ static avi_t *AVI_init(demux_avi_t *this) {
       continue;
     }
 
-    n = LE_32(hdrl_data + i + 4);
+    n = _X_LE_32(hdrl_data + i + 4);
     n = PAD_EVEN(n);
 
     /* Interpret the tag and its args */
@@ -863,10 +858,10 @@ static avi_t *AVI_init(demux_avi_t *this) {
       if(strncasecmp(hdrl_data + i, "vids", 4) == 0 && !vids_strh_seen) {
 
         AVI->compressor = *(uint32_t *) (hdrl_data + i + 4);
-        AVI->dwInitialFrames = LE_32(hdrl_data + i + 16);
-        AVI->dwScale         = LE_32(hdrl_data + i + 20);
-        AVI->dwRate          = LE_32(hdrl_data + i + 24);
-        AVI->dwStart         = LE_32(hdrl_data + i + 28);
+        AVI->dwInitialFrames = _X_LE_32(hdrl_data + i + 16);
+        AVI->dwScale         = _X_LE_32(hdrl_data + i + 20);
+        AVI->dwRate          = _X_LE_32(hdrl_data + i + 24);
+        AVI->dwStart         = _X_LE_32(hdrl_data + i + 28);
 
         if(AVI->dwScale!=0)
           AVI->fps = (double)AVI->dwRate/(double)AVI->dwScale;
@@ -891,15 +886,15 @@ static avi_t *AVI_init(demux_avi_t *this) {
           AVI->audio[AVI->n_audio] = a;
 
           a->audio_strn      = num_stream;
-          a->dwInitialFrames = LE_32(hdrl_data + i + 16);
-          a->dwScale         = LE_32(hdrl_data + i + 20);
-          a->dwRate          = LE_32(hdrl_data + i + 24);
-          a->dwStart         = LE_32(hdrl_data + i + 28);
+          a->dwInitialFrames = _X_LE_32(hdrl_data + i + 16);
+          a->dwScale         = _X_LE_32(hdrl_data + i + 20);
+          a->dwRate          = _X_LE_32(hdrl_data + i + 24);
+          a->dwStart         = _X_LE_32(hdrl_data + i + 28);
           
           lprintf("dwScale=%d, dwRate=%d, dwInitialFrames=%d, dwStart=%d, num_stream=%d\n",
                   a->dwScale, a->dwRate, a->dwInitialFrames, a->dwStart, num_stream);
           
-          a->dwSampleSize  = LE_32(hdrl_data + i + 44);
+          a->dwSampleSize  = _X_LE_32(hdrl_data + i + 44);
           a->audio_tot     = 0;
           auds_strh_seen   = 1;
           lprintf("audio stream header, num_stream=%d\n", num_stream);
@@ -914,7 +909,7 @@ static avi_t *AVI_init(demux_avi_t *this) {
       num_stream++;
       
     } else if(strncasecmp(hdrl_data+i,"dmlh",4) == 0) {
-      AVI->total_frames = LE_32(hdrl_data+i+8);
+      AVI->total_frames = _X_LE_32(hdrl_data+i+8);
 #ifdef DEBUG_ODML
       lprintf( "AVI: real number of frames %d\n", AVI->total_frames);
 #endif
@@ -922,7 +917,7 @@ static avi_t *AVI_init(demux_avi_t *this) {
 	  
     } else if(strncasecmp(hdrl_data + i, "strf", 4) == 0) {
       i += 4;
-      strf_size = LE_32(hdrl_data + i);
+      strf_size = _X_LE_32(hdrl_data + i);
       i += 4;
       if(lasttag == 1) {
         /* lprintf ("size : %d\n",sizeof(AVI->bih)); */
@@ -954,7 +949,7 @@ static avi_t *AVI_init(demux_avi_t *this) {
 
         lprintf ("palette_count: %d\n", AVI->palette_count);
         if (AVI->palette_count > 256) {
-          lprintf ("number of colors exceeded 256 (%d)", AVI->palette_count);
+          lprintf ("number of colours exceeded 256 (%d)", AVI->palette_count);
           AVI->palette_count = 256;
         }
         if ((strf_size - sizeof(xine_bmiheader)) >= (AVI->palette_count * 4)) {
@@ -1001,11 +996,11 @@ static avi_t *AVI_init(demux_avi_t *this) {
       superindex = (avisuperindex_chunk *) malloc (sizeof (avisuperindex_chunk));
       a = hdrl_data + i;
       memcpy (superindex->fcc, a, 4);             a += 4;
-      superindex->dwSize = LE_32(a);              a += 4;
-      superindex->wLongsPerEntry = LE_16(a);      a += 2;
+      superindex->dwSize = _X_LE_32(a);              a += 4;
+      superindex->wLongsPerEntry = _X_LE_16(a);      a += 2;
       superindex->bIndexSubType = *a;             a += 1;
       superindex->bIndexType = *a;                a += 1;
-      superindex->nEntriesInUse = LE_32(a);       a += 4;
+      superindex->nEntriesInUse = _X_LE_32(a);       a += 4;
       memcpy (superindex->dwChunkId, a, 4);       a += 4;
 
 #ifdef DEBUG_ODML
@@ -1039,9 +1034,9 @@ static avi_t *AVI_init(demux_avi_t *this) {
       superindex->aIndex = malloc (superindex->nEntriesInUse * sizeof (avisuperindex_entry));
       /* position of ix## chunks */
       for (j = 0; j < superindex->nEntriesInUse; ++j) {
-        superindex->aIndex[j].qwOffset = LE_64 (a);   a += 8;
-        superindex->aIndex[j].dwSize = LE_32 (a);     a += 4;
-        superindex->aIndex[j].dwDuration = LE_32 (a); a += 4;
+        superindex->aIndex[j].qwOffset = _X_LE_64 (a);   a += 8;
+        superindex->aIndex[j].dwSize = _X_LE_32 (a);     a += 4;
+        superindex->aIndex[j].dwDuration = _X_LE_32 (a); a += 4;
 #ifdef DEBUG_ODML
         printf("[%d] 0x%llx 0x%x %u\n", j,
                (uint64_t)superindex->aIndex[j].qwOffset,
@@ -1149,21 +1144,21 @@ static avi_t *AVI_init(demux_avi_t *this) {
         ERR_EXIT(AVI_ERR_NO_VIDS);
       }
 
-      pos = LE_32(AVI->idx[i] + 8);
-      len = LE_32(AVI->idx[i] + 12);
+      pos = _X_LE_32(AVI->idx[i] + 8);
+      len = _X_LE_32(AVI->idx[i] + 12);
 
       this->input->seek(this->input, pos, SEEK_SET);
       if(this->input->read(this->input, data, 8) != 8)
         ERR_EXIT(AVI_ERR_READ) ;
 
-      if( (strncasecmp(data, AVI->idx[i], 4) == 0) && (LE_32(data + 4) == len) ) {
+      if( (strncasecmp(data, AVI->idx[i], 4) == 0) && (_X_LE_32(data + 4) == len) ) {
         idx_type = 1; /* Index from start of file */
       } else {
 
         this->input->seek(this->input, pos + AVI->movi_start - 4, SEEK_SET);
         if(this->input->read(this->input, data, 8) != 8)
           ERR_EXIT(AVI_ERR_READ) ;
-        if( strncasecmp(data,AVI->idx[i], 4) == 0 && LE_32(data + 4) == len ) {
+        if( strncasecmp(data,AVI->idx[i], 4) == 0 && _X_LE_32(data + 4) == len ) {
           idx_type = 2; /* Index from start of movi list */
         }
       }
@@ -1184,9 +1179,9 @@ static avi_t *AVI_init(demux_avi_t *this) {
 
       if((AVI->idx[i][0] == AVI->video_tag[0]) &&
          (AVI->idx[i][1] == AVI->video_tag[1])) {
-        off_t pos = LE_32(AVI->idx[i] + 8) + ioff;
-        uint32_t len = LE_32(AVI->idx[i] + 12);
-        uint32_t flags = LE_32(AVI->idx[i] + 4);
+        off_t pos = _X_LE_32(AVI->idx[i] + 8) + ioff;
+        uint32_t len = _X_LE_32(AVI->idx[i] + 12);
+        uint32_t flags = _X_LE_32(AVI->idx[i] + 4);
 
         if (video_index_append(AVI, pos, len, flags) == -1) {
           ERR_EXIT(AVI_ERR_NO_MEM) ;
@@ -1197,8 +1192,8 @@ static avi_t *AVI_init(demux_avi_t *this) {
 
           if((AVI->idx[i][0] == audio->audio_tag[0]) &&
              (AVI->idx[i][1] == audio->audio_tag[1])) {
-            off_t pos = LE_32(AVI->idx[i] + 8) + ioff;
-            uint32_t len = LE_32(AVI->idx[i] + 12);
+            off_t pos = _X_LE_32(AVI->idx[i] + 8) + ioff;
+            uint32_t len = _X_LE_32(AVI->idx[i] + 12);
 
             /* VBR streams (hack from mplayer) */
             if (audio->wavex && audio->wavex->nBlockAlign) {
@@ -1256,11 +1251,11 @@ static avi_t *AVI_init(demux_avi_t *this) {
              continue;
           }
 
-          nrEntries = LE_32(en + 12);
+          nrEntries = _X_LE_32(en + 12);
   #ifdef DEBUG_ODML
           printf("[%d:0] Video nrEntries %ld\n", j, (long)nrEntries);
   #endif
-          offset = LE_64(en + 20);
+          offset = _X_LE_64(en + 20);
 
           /* skip header */
           en += hdrl_len;
@@ -1271,7 +1266,7 @@ static avi_t *AVI_init(demux_avi_t *this) {
             uint32_t len;
             uint32_t flags;
 
-            pos = offset + LE_32(en); en += 4;
+            pos = offset + _X_LE_32(en); en += 4;
             len = odml_len(en);
             flags = odml_key(en); en += 4;
             video_index_append(AVI, pos, len, flags);
@@ -1326,11 +1321,11 @@ static avi_t *AVI_init(demux_avi_t *this) {
             continue;
           }
 
-          nrEntries = LE_32(en + 12);
+          nrEntries = _X_LE_32(en + 12);
 #ifdef DEBUG_ODML
           /*printf("[%d:%d] Audio nrEntries %ld\n", j, audtr, nrEntries); */
 #endif
-          offset = LE_64(en + 20);
+          offset = _X_LE_64(en + 20);
 
           /* skip header */
           en += hdrl_len;
@@ -1341,7 +1336,7 @@ static avi_t *AVI_init(demux_avi_t *this) {
             off_t pos;
             uint32_t len;
 
-            pos = offset + LE_32(en); en += 4;
+            pos = offset + _X_LE_32(en); en += 4;
             len = odml_len(en); en += 4;
           
             /* VBR streams (hack from mplayer) */
@@ -1392,7 +1387,7 @@ static avi_t *AVI_init(demux_avi_t *this) {
   AVI->video_posf = 0;
   AVI->video_posb = 0;
 
-  lprintf("done, pos=%lld, AVI->movi_start=%" PRIdMAX "\n", this->input->get_current_pos(this->input), (intmax_t)AVI->movi_start);
+  lprintf("done, pos=%"PRId64", AVI->movi_start=%" PRIdMAX "\n", this->input->get_current_pos(this->input), (intmax_t)AVI->movi_start);
   return AVI;
 }
 
@@ -1648,7 +1643,7 @@ static int get_chunk_header(demux_avi_t *this, uint32_t *len, int *audio_stream)
   while (1) {
     if (this->input->read(this->input, data, AVI_HEADER_SIZE) != AVI_HEADER_SIZE)
       break;
-    *len = LE_32(data + 4);
+    *len = _X_LE_32(data + 4);
 
     lprintf("header: %c%c%c%c, pos=%" PRIdMAX ", len=%u\n",
             data[0], data[1], data[2], data[3],
