@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2004 the xine project
+ * Copyright (C) 2000-2007 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -15,15 +15,13 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  */
 
 /*
  * FLAC File Demuxer by Mike Melanson (melanson@pcisys.net)
  * For more information on the FLAC file format, visit:
  *   http://flac.sourceforge.net/
- *
- * $Id: demux_flac.c,v 1.13 2007/01/19 00:26:40 dgp85 Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -49,6 +47,7 @@
 #include "bswap.h"
 #include "group_audio.h"
 
+#include "id3.h"
 #include "flacutils.h"
 
 typedef struct {
@@ -66,10 +65,10 @@ typedef struct {
   off_t                data_start;
   off_t                data_size;
 
-  unsigned char        streaminfo[sizeof(xine_waveformatex) + FLAC_STREAMINFO_SIZE];
   flac_seekpoint_t    *seekpoints;
   int                  seekpoint_count;
 
+  unsigned char        streaminfo[sizeof(xine_waveformatex) + FLAC_STREAMINFO_SIZE];
 } demux_flac_t;
 
 typedef struct {
@@ -81,7 +80,7 @@ typedef struct {
  * It returns 1 if flac file was opened successfully. */
 static int open_flac_file(demux_flac_t *flac) {
 
-  unsigned char preamble[4];
+  unsigned char preamble[10];
   unsigned int block_length;
   unsigned char buffer[FLAC_SEEKPOINT_SIZE];
   unsigned char *streaminfo = flac->streaminfo + sizeof(xine_waveformatex);
@@ -89,52 +88,30 @@ static int open_flac_file(demux_flac_t *flac) {
 
   flac->seekpoints = NULL;
 
-  /* fetch the file signature */
+  /* fetch the file signature, 4 bytes will read both the fLaC
+   * signature and the */
   if (_x_demux_read_header(flac->input, preamble, 4) != 4)
     return 0;
 
+  flac->input->seek(flac->input, 4, SEEK_SET);
+
+  /* Unfortunately some FLAC files have an ID3 flag prefixed on them
+   * before the actual FLAC headers... these are barely legal, but
+   * users use them and want them working, so check and skip the ID3
+   * tag if present.
+   */
+  if ( id3v2_istag(preamble) ) {
+    id3v2_parse_tag(flac->input, flac->stream, preamble);
+
+    if ( flac->input->read(flac->input, preamble, 4) != 4 )
+      return 0;
+  }
+
   /* validate signature */
-  if ((preamble[0] != 'f') ||
-      (preamble[1] != 'L') ||
-      (preamble[2] != 'a') ||
-      (preamble[3] != 'C')) {
-
-    uint32_t id3size;
-
-    /* Unfortunately some FLAC files have an ID3 flag prefixed on them
-     * before the actual FLAC headers... these are barely legal, but
-     * users use them and want them working, so check and skip the ID3
-     * tag if present.
-     */
-    if ( preamble[0] != 'I' || preamble[1] != 'D' || preamble[2] != '3' )
+  if ((preamble[0] != 'f') || (preamble[1] != 'L') ||
+      (preamble[2] != 'a') || (preamble[3] != 'C'))
       return 0;
 
-    /* First 3 bytes are the ID3 signature as above, then comes two bytes
-     * encoding the major and minor version of ID3 used, that we can ignore
-     * as long as we don't try to read the metadata; after those there's a
-     * single byte with flags that depends on the ID3 version used; and now
-     * after all that stuff, there's the size of the rest of the tag, which
-     * is encoded as four bytes.. but only 7 out of 8 bits of every byte is
-     * used... don't ask.
-     */
-    flac->input->seek(flac->input, 6, SEEK_SET);
-    if ( flac->input->read(flac->input, preamble, 4) != 4 )
-      return 0;
-
-    id3size = (preamble[0] << 7*3) + (preamble[1] << 7*2) + (preamble[2] << 7) + preamble[3];
-    
-    flac->input->seek(flac->input, id3size, SEEK_CUR);
-
-    if ( flac->input->read(flac->input, preamble, 4) != 4 )
-      return 0;
-    
-    if ( preamble[0] != 'f' || preamble[1] != 'L' || preamble[2] != 'a' || preamble[3] != 'C' )
-      return 0;
-      
-  } else 
-    /* file is qualified; skip over the signature bytes in the stream */
-    flac->input->seek(flac->input, 4, SEEK_SET);
-
   /* loop through the metadata blocks; use a do-while construct since there
    * will always be 1 metadata block */
   do {
@@ -161,12 +138,12 @@ static int open_flac_file(demux_flac_t *flac) {
         flac->streaminfo + sizeof(xine_waveformatex),
         FLAC_STREAMINFO_SIZE) != FLAC_STREAMINFO_SIZE)
         return 0;
-      flac->sample_rate = BE_32(&streaminfo[10]);
+      flac->sample_rate = _X_BE_32(&streaminfo[10]);
       flac->channels = ((flac->sample_rate >> 9) & 0x07) + 1;
       flac->bits_per_sample = ((flac->sample_rate >> 4) & 0x1F) + 1;
       flac->sample_rate >>= 12;
-      flac->total_samples = BE_64(&streaminfo[10]) & UINT64_C(0x0FFFFFFFFF);  /* 36 bits */
-      lprintf ("%d Hz, %d bits, %d channels, %lld total samples\n", 
+      flac->total_samples = _X_BE_64(&streaminfo[10]) & UINT64_C(0x0FFFFFFFFF);  /* 36 bits */
+      lprintf ("%d Hz, %d bits, %d channels, %"PRId64" total samples\n", 
         flac->sample_rate, flac->bits_per_sample, 
         flac->channels, flac->total_samples);
       break;
@@ -192,16 +169,16 @@ static int open_flac_file(demux_flac_t *flac) {
       for (i = 0; i < flac->seekpoint_count; i++) {
         if (flac->input->read(flac->input, buffer, FLAC_SEEKPOINT_SIZE) != FLAC_SEEKPOINT_SIZE)
           return 0;
-        flac->seekpoints[i].sample_number = BE_64(&buffer[0]);
-        lprintf (" %d: sample %lld, ", i, flac->seekpoints[i].sample_number);
-        flac->seekpoints[i].offset = BE_64(&buffer[8]);
-        flac->seekpoints[i].size = BE_16(&buffer[16]);
-        lprintf ("@ 0x%llX, size = %d bytes, ", 
+        flac->seekpoints[i].sample_number = _X_BE_64(&buffer[0]);
+        lprintf (" %d: sample %"PRId64", ", i, flac->seekpoints[i].sample_number);
+        flac->seekpoints[i].offset = _X_BE_64(&buffer[8]);
+        flac->seekpoints[i].size = _X_BE_16(&buffer[16]);
+        lprintf ("@ 0x%"PRIX64", size = %d bytes, ", 
           flac->seekpoints[i].offset, flac->seekpoints[i].size);
         flac->seekpoints[i].pts = flac->seekpoints[i].sample_number;
         flac->seekpoints[i].pts *= 90000;
         flac->seekpoints[i].pts /= flac->sample_rate;
-        lprintf ("pts = %lld\n", flac->seekpoints[i].pts);
+        lprintf ("pts = %"PRId64"\n", flac->seekpoints[i].pts);
       }
       break;
 
@@ -213,7 +190,7 @@ static int open_flac_file(demux_flac_t *flac) {
       lprintf ("VORBIS_COMMENT metadata\n");
       {
         char comments[block_length];
-        void *ptr = comments;
+        char *ptr = comments;
         uint32_t length, user_comment_list_length;
         int cn;
         char *comment;
@@ -223,15 +200,15 @@ static int open_flac_file(demux_flac_t *flac) {
           int tracknumber = -1;
           int tracktotal = -1;
 
-          length = LE_32(ptr);
+          length = _X_LE_32(ptr);
           ptr += 4 + length;
 
-          user_comment_list_length = LE_32(ptr);
+          user_comment_list_length = _X_LE_32(ptr);
           ptr += 4;
 
           cn = 0;
           for (; cn < user_comment_list_length; cn++) {
-            length = LE_32(ptr);
+            length = _X_LE_32(ptr);
             ptr += 4;
 
             comment = (char*) ptr;
@@ -567,7 +544,8 @@ static const char *get_extensions (demux_class_t *this_gen) {
 }
 
 static const char *get_mimetypes (demux_class_t *this_gen) {
-  return NULL;
+  return "audio/x-flac: flac: FLAC Audio;"
+    "audio/flac: flac: FLAC Audio;";
 }
 
 static void class_dispose (demux_class_t *this_gen) {
