@@ -15,13 +15,10 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * EBML parser
  * a lot of ideas from the gstreamer parser
- *
- * $Id: ebml.c,v 1.4 2005/11/28 12:24:57 valtri Exp $
- *
  */
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -297,12 +294,12 @@ int ebml_read_float (ebml_parser_t *ebml, ebml_elem_t *elem, double *num) {
   if (size == 4) {
     float f;
 
-    *((uint32_t *) &f) = BE_32(data);
+    *((uint32_t *) &f) = _X_BE_32(data);
     *num = f;
   } else {
     double d;
 
-    *((uint64_t *) &d) = BE_64(data);
+    *((uint64_t *) &d) = _X_BE_64(data);
     *num = d;
   }
   return 1;
