@@ -17,7 +17,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * pvr input plugin for WinTV-PVR 250/350 pci cards using driver from:
  *   http://ivtv.sf.net
@@ -37,8 +37,6 @@
  *
  * usage: 
  *   xine pvr:/<prefix_to_tmp_files>\!<prefix_to_saved_files>\!<max_page_age>
- *
- * $Id: input_pvr.c,v 1.56 2005/10/14 21:02:16 miguelfreitas Exp $
  */
 
 /**************************************************************************
@@ -579,7 +577,7 @@ static int pvr_break_rec_page (pvr_input_plugin_t *this) {
   
   char *filename;
   
-  if( this->session == -1 ) /* not recording */
+  if( this->session == (unsigned)-1 ) /* not recording */
     return 1;
      
   if( this->rec_fd != -1 && this->rec_fd != this->play_fd ) {
@@ -607,9 +605,9 @@ static int pvr_break_rec_page (pvr_input_plugin_t *this) {
   free(filename);
      
   /* erase first_page if old and not to be saved */
-  if( this->max_page_age != -1 && 
+  if( this->max_page_age != (unsigned)-1 && 
       this->rec_page - this->max_page_age == this->first_page &&
-      (this->save_page == -1 || this->first_page < this->save_page) ) {
+      (this->save_page == (unsigned)-1 || this->first_page < this->save_page) ) {
     
     filename = make_temp_name(this, this->first_page);
 
@@ -635,7 +633,7 @@ static int pvr_rec_file(pvr_input_plugin_t *this) {
   
   off_t pos;
 
-  if( this->session == -1 ) /* not recording */
+  if( this->session == (unsigned)-1 ) /* not recording */
     return 1;
   
   /* check if it's time to change page/file */
@@ -646,7 +644,7 @@ static int pvr_rec_file(pvr_input_plugin_t *this) {
   pos = (off_t)(this->rec_blk - this->page_block[this->rec_page]) * PVR_BLOCK_SIZE;
   if( lseek (this->rec_fd, pos, SEEK_SET) != pos ) {
     xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, 
-	    "input_pvr: error setting position for writing %lld\n", pos);
+	    "input_pvr: error setting position for writing %" PRIdMAX "\n", (intmax_t)pos);
     return 0;
   }
   if( this->rec_fd != -1 ) {
@@ -772,7 +770,7 @@ static int pvr_play_file(pvr_input_plugin_t *this, fifo_buffer_t *fifo, uint8_t
       pos = (off_t)(this->play_blk - this->page_block[this->play_page]) * PVR_BLOCK_SIZE;
       if( lseek (this->play_fd, pos, SEEK_SET) != pos ) {
         xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
-		"input_pvr: error setting position for reading %lld\n", pos);
+		"input_pvr: error setting position for reading %" PRIdMAX "\n", (intmax_t)pos);
         return 0;
       }
       if( read(this->play_fd, buffer, PVR_BLOCK_SIZE) < PVR_BLOCK_SIZE ) {
@@ -907,7 +905,7 @@ static void pvr_finish_recording (pvr_input_plugin_t *this) {
       
       src_filename = make_temp_name(this, i);
       
-      if( this->save_page == -1 || i < this->save_page ) {
+      if( this->save_page == (unsigned)-1 || i < this->save_page ) {
         lprintf("erasing old pvr file (%s)\n", src_filename);
 
         remove(src_filename);
@@ -926,7 +924,7 @@ static void pvr_finish_recording (pvr_input_plugin_t *this) {
       free(src_filename);
     }
     
-    if( this->save_page != -1 && (!this->save_name || !strlen(this->save_name)) ) {
+    if( this->save_page != (unsigned)-1 && (!this->save_name || !strlen(this->save_name)) ) {
       saved_show_t        *show = malloc(sizeof(saved_show_t));
       xine_event_t         event;
       xine_pvr_save_data_t data;
@@ -934,7 +932,7 @@ static void pvr_finish_recording (pvr_input_plugin_t *this) {
       show->base_name = save_base;
       show->id = ++this->saved_id;
       show->pages = this->rec_page - this->save_page + 1;
-      xine_list_append_content (this->saved_shows, show);
+      xine_list_push_back (this->saved_shows, show);
       
       lprintf("sending event with base name [%s]\n", show->base_name);
 
@@ -989,69 +987,92 @@ static void pvr_event_handler (pvr_input_plugin_t *this) {
       /* make sure we are not paused */
       _x_set_speed(this->stream, XINE_SPEED_NORMAL);
 
-      if( v4l2_data->session_id != this->session ) {
-        /* if session changes -> closes the old one */
-        pthread_mutex_lock(&this->lock);
-        pvr_finish_recording(this);
-        time(&this->start_time);
-        this->show_time = this->start_time;
-        this->session = v4l2_data->session_id;
-        this->new_session = 1;
-        this->pvr_play_paused = 0;
-        this->scr_tunning = 0;
-        pvrscr_speed_tunning(this->scr, 1.0 );
-        pvr_break_rec_page(this);
-        pthread_mutex_unlock(&this->lock);
-        _x_demux_flush_engine (this->stream);
-      } else {
-        /* no session change, break the page and store a new show_time */
-        pthread_mutex_lock(&this->lock);
-        pvr_break_rec_page(this);
-        this->show_page = this->rec_page;
-        pthread_mutex_unlock(&this->lock);
-        time(&this->show_time);
+      if ( v4l2_data->session_id != -1) {
+        if( v4l2_data->session_id != this->session ) {
+          /* if session changes -> closes the old one */
+          pthread_mutex_lock(&this->lock);
+          pvr_finish_recording(this);
+          time(&this->start_time);
+          this->show_time = this->start_time;
+          this->session = v4l2_data->session_id;
+          this->new_session = 1;
+          this->pvr_play_paused = 0;
+          this->scr_tunning = 0;
+          pvrscr_speed_tunning(this->scr, 1.0 );
+          pvr_break_rec_page(this);
+          pthread_mutex_unlock(&this->lock);
+          _x_demux_flush_engine (this->stream);
+        } else {
+          /* no session change, break the page and store a new show_time */
+          pthread_mutex_lock(&this->lock);
+          pvr_break_rec_page(this);
+          this->show_page = this->rec_page;
+          pthread_mutex_unlock(&this->lock);
+          time(&this->show_time);
+        }
       }
-      
-      if( v4l2_data->input != this->input ||
-          v4l2_data->channel != this->channel || 
-          v4l2_data->frequency != this->frequency ) {
-        struct v4l2_frequency vf;
 
+      pthread_mutex_lock(&this->dev_lock);
+
+      /* change input */
+      if (v4l2_data->input != -1 && v4l2_data->input != this->input) {
         this->input = v4l2_data->input;
-        this->channel = v4l2_data->channel;
-        this->frequency = v4l2_data->frequency;
 
-        lprintf("switching to input:%d chan:%d freq:%.2f\n", 
-		v4l2_data->input, 
-		v4l2_data->channel,
-		(float)v4l2_data->frequency * 62.5);
-  
-        pthread_mutex_lock(&this->dev_lock);
-        if( ioctl(this->dev_fd, VIDIOC_S_INPUT, &this->input) )
-          xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, 
-		  "input_pvr: error setting v4l2 input\n");
-          
-        vf.frequency = this->frequency;
-        vf.tuner = 0;
-        if( ioctl(this->dev_fd, VIDIOC_S_FREQUENCY, &vf) )
-          xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, 
-		  "input_pvr: error setting v4l2 frequency\n");
-
-        /* workaround an ivtv bug where stream gets bad mpeg2 artifacts
-         * after changing inputs. reopening the device fixes it.
-         */
+        /* as of ivtv 0.10.6: must close and reopen to set input */
         close(this->dev_fd);
         this->dev_fd = open (this->class->devname, O_RDWR);
-        if (this->dev_fd == -1) {
-          xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, 
-		  "input_pvr: error opening device %s\n", this->class->devname );
-          return;
+        if (this->dev_fd < 0) {
+          xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
+                  "input_pvr: error opening device %s\n", this->class->devname );
+        } else {
+          if( ioctl(this->dev_fd, VIDIOC_S_INPUT, &this->input) == 0 ) {
+            lprintf("Tuner Input set to:%d\n", v4l2_data->input);
+          } else {
+            xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, 
+                    "input_pvr: error setting v4l2 input\n");
+          }
         }
-        pthread_mutex_unlock(&this->dev_lock);
-        
-        /* FIXME: also flush the device */
-        /* _x_demux_flush_engine(this->stream); */
       }
+
+      /* change channel */          
+      if (v4l2_data->channel != -1 && v4l2_data->channel != this->channel) {
+        lprintf("change channel to:%d\n", v4l2_data->channel);
+        this->channel = v4l2_data->channel;
+      }
+
+      /* change frequency */
+      if (v4l2_data->frequency != -1 && v4l2_data->frequency != this->frequency) {
+        double freq = (double)v4l2_data->frequency / 1000.0;
+        struct v4l2_frequency vf;
+        struct v4l2_tuner vt;
+        double fac = 16;
+
+        memset(&vf, 0, sizeof(vf));
+        memset(&vt, 0, sizeof(vt));
+
+        this->frequency = v4l2_data->frequency;
+
+        if (ioctl(this->dev_fd, VIDIOC_G_TUNER, &vt) == 0) {
+          fac = (vt.capability & V4L2_TUNER_CAP_LOW) ? 16000 : 16;
+        }
+
+        vf.tuner = 0;
+        vf.type = vt.type;
+        vf.frequency = (__u32)(freq * fac);
+
+        if (ioctl(this->dev_fd, VIDIOC_S_FREQUENCY, &vf) == 0) {
+          lprintf("Tuner Frequency set to %d (%f.3 MHz)\n", vf.frequency, vf.frequency / fac);
+        } else {
+          xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
+                  "input_pvr: error setting v4l2 frequency\n");
+        }
+      }
+
+      pthread_mutex_unlock(&this->dev_lock);
+        
+      /* FIXME: also flush the device */
+      /* _x_demux_flush_engine(this->stream); */
+
       break;
     
     
@@ -1098,11 +1119,13 @@ static void pvr_event_handler (pvr_input_plugin_t *this) {
           char *src_filename;
           char *dst_filename;
           saved_show_t  *show;
+	  xine_list_iterator_t ite;
             
           pthread_mutex_lock(&this->lock);
   
-          show = xine_list_first_content (this->saved_shows);
-          while (show) {
+          ite = xine_list_front (this->saved_shows);
+          while (ite) {
+            show = xine_list_get_value(this->saved_shows, ite);
             if( show->id == save_data->id ) {
               int i;
                 
@@ -1117,12 +1140,12 @@ static void pvr_event_handler (pvr_input_plugin_t *this) {
                 free(dst_filename);
                 free(src_filename);
               }
-              xine_list_delete_current (this->saved_shows);
+              xine_list_remove (this->saved_shows, ite);
               free (show->base_name);
               free (show);
               break;
             }
-            show = xine_list_next_content (this->saved_shows);
+            ite = xine_list_next (this->saved_shows, ite);
           }
 
           pthread_mutex_unlock(&this->lock);
@@ -1315,7 +1338,7 @@ static uint32_t pvr_plugin_get_blocksize (input_plugin_t *this_gen) {
   return PVR_BLOCK_SIZE;
 }
 
-static char* pvr_plugin_get_mrl (input_plugin_t *this_gen) {
+static const char* pvr_plugin_get_mrl (input_plugin_t *this_gen) {
   pvr_input_plugin_t *this = (pvr_input_plugin_t *) this_gen;
 
   return this->mrl;
@@ -1331,6 +1354,7 @@ static void pvr_plugin_dispose (input_plugin_t *this_gen ) {
   pvr_input_plugin_t *this = (pvr_input_plugin_t *) this_gen;
   void          *p;
   saved_show_t  *show;
+  xine_list_iterator_t ite;
 
   if( this->pvr_running ) {
     lprintf("finishing pvr thread\n");
@@ -1366,13 +1390,14 @@ static void pvr_plugin_dispose (input_plugin_t *this_gen ) {
   if (this->save_prefix)
     free (this->save_prefix);
   
-  show = xine_list_first_content (this->saved_shows);
-  while (show) {
+  ite = xine_list_front (this->saved_shows);
+  while (ite) {
+    show = xine_list_get_value(this->saved_shows, ite);
     free (show->base_name);
     free (show);
-    show = xine_list_next_content (this->saved_shows);
+    ite = xine_list_next (this->saved_shows, ite);
   }
-  xine_list_free(this->saved_shows);
+  xine_list_delete(this->saved_shows);
   free (this);
 }
 
@@ -1520,7 +1545,7 @@ static input_plugin_t *pvr_class_get_instance (input_class_t *cls_gen, xine_stre
  * plugin class functions
  */
 
-static char *pvr_class_get_description (input_class_t *this_gen) {
+static const char *pvr_class_get_description (input_class_t *this_gen) {
   return _("WinTV-PVR 250/350 input plugin");
 }
 
@@ -1544,9 +1569,9 @@ static void *init_plugin (xine_t *xine, void *data) {
   this->xine   = xine;
   this->config = xine->config;
 
-  this->devname = this->config->register_string(this->config,
+  this->devname = this->config->register_filename(this->config,
 				    "media.wintv_pvr.device",
-				    PVR_DEVICE,
+				    PVR_DEVICE, XINE_CONFIG_STRING_IS_DEVICE_NAME,
 				    _("device used for WinTV-PVR 250/350 (pvr plugin)"),
 				    _("The path to the device of your WinTV card."),
 				    10, NULL,
@@ -1567,7 +1592,7 @@ static void *init_plugin (xine_t *xine, void *data) {
  * exported plugin catalog entry
  */
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */  
   { PLUGIN_INPUT | PLUGIN_MUST_PRELOAD, 17, "pvr", XINE_VERSION_CODE, NULL, init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
