@@ -15,9 +15,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: image.c,v 1.17 2005/02/03 23:18:34 holstsn Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * a image video decoder
  */
@@ -37,14 +35,21 @@
 #define LOG
 */
 
+#include <wand/magick_wand.h>
+#ifdef PACKAGE_NAME
+#undef PACKAGE_BUGREPORT
+#undef PACKAGE_NAME
+#undef PACKAGE_STRING
+#undef PACKAGE_TARNAME
+#undef PACKAGE_VERSION
+#endif
+
 #include "xine_internal.h"
 #include "video_out.h"
 #include "buffer.h"
 #include "xineutils.h"
 #include "bswap.h"
 
-#include <wand/magick_wand.h>
-
 typedef struct {
   video_decoder_class_t   decoder_class;
 
@@ -74,7 +79,7 @@ static void image_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
 
   if (!this->video_open) {
     lprintf("opening video\n");
-    this->stream->video_out->open(this->stream->video_out, this->stream);
+    (this->stream->video_out->open) (this->stream->video_out, this->stream);
     this->video_open = 1;
   }
 
@@ -273,12 +278,12 @@ static void *init_class (xine_t *xine, void *data) {
 static uint32_t supported_types[] = { BUF_VIDEO_IMAGE,
                                       0 };
 
-static decoder_info_t dec_info_image = {
+static const decoder_info_t dec_info_image = {
   supported_types,     /* supported types */
   6                    /* priority        */
 };
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */  
   { PLUGIN_VIDEO_DECODER, 18, "image", XINE_VERSION_CODE, &dec_info_image, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
