@@ -201,8 +201,8 @@ static inline v2g zoomVector(ZoomFilterFXWrapperData *data, float X, float Y)
     /* Noise */
     if (data->noisify)
     {
-        vx += (((float)random()) / ((float)RAND_MAX) - 0.5f) / 50.0f;
-        vy += (((float)random()) / ((float)RAND_MAX) - 0.5f) / 50.0f;
+        vx += (((float)rand()) / ((float)RAND_MAX) - 0.5f) / 50.0f;
+        vy += (((float)rand()) / ((float)RAND_MAX) - 0.5f) / 50.0f;
     }
     
     /* Hypercos */
@@ -740,7 +740,7 @@ static void zoomFilterVisualFXWrapper_apply (struct _VISUAL_FX *_this, Pixel *sr
 
 VisualFX zoomFilterVisualFXWrapper_create(void)
 {
-    VisualFX fx;
+    VisualFX fx = {0};
     fx.init = zoomFilterVisualFXWrapper_init;
     fx.free = zoomFilterVisualFXWrapper_free;
     fx.apply = zoomFilterVisualFXWrapper_apply;
