@@ -15,9 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: video_out_sdl.c,v 1.47 2006/06/02 22:18:58 dsalt Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * video_out_sdl.c, Simple DirectMedia Layer
  *
@@ -362,6 +360,14 @@ static int sdl_get_property (vo_driver_t *this_gen, int property) {
       return this->sc.gui_width;
     case VO_PROP_WINDOW_HEIGHT:
       return this->sc.gui_height;
+    case VO_PROP_OUTPUT_WIDTH:
+      return this->sc.output_width;
+    case VO_PROP_OUTPUT_HEIGHT:
+      return this->sc.output_height;
+    case VO_PROP_OUTPUT_XOFFSET:
+      return this->sc.output_xoffset;
+    case VO_PROP_OUTPUT_YOFFSET:
+      return this->sc.output_yoffset;
     case VO_PROP_ASPECT_RATIO:
     return this->sc.user_ratio;
   }
@@ -608,7 +614,7 @@ static const vo_info_t vo_info_sdl = {
   XINE_VISUAL_TYPE_X11, /* visual type supported by this plugin */
 };
 
-const plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_VIDEO_OUT, 21, "sdl", XINE_VERSION_CODE, &vo_info_sdl, init_class },
   { PLUGIN_NONE, 0, "" , 0 , NULL, NULL}
