@@ -15,9 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: video_out_xcbxv.c,v 1.6 2007/03/29 18:58:21 dgp85 Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * video_out_xcbxv.c, X11 video extension interface for xine
  *
@@ -213,11 +211,6 @@ static void create_ximage(xv_driver_t *this, xv_frame_t *frame, int width, int h
 
   unsigned int length;
 
-  if (width <= 0)
-    width = 1;
-  if (height <= 0)
-    height = 1;
-
   if (this->use_pitch_alignment) {
     width = (width + 7) & ~0x7;
   }
@@ -773,6 +766,18 @@ static int xv_get_property (vo_driver_t *this_gen, int property) {
     case VO_PROP_WINDOW_HEIGHT:
       this->props[property].value = this->sc.gui_height;
       break;
+    case VO_PROP_OUTPUT_WIDTH:
+      this->props[property].value = this->sc.output_width;
+      break;
+    case VO_PROP_OUTPUT_HEIGHT:
+      this->props[property].value = this->sc.output_height;
+      break;
+    case VO_PROP_OUTPUT_XOFFSET:
+      this->props[property].value = this->sc.output_xoffset;
+      break;
+    case VO_PROP_OUTPUT_YOFFSET:
+      this->props[property].value = this->sc.output_yoffset;
+      break;
   }
 
   lprintf("video_out_xcbxv: property #%d = %d\n", property, this->props[property].value);
@@ -996,28 +1001,30 @@ static int xv_gui_data_exchange (vo_driver_t *this_gen,
 }
 
 static void xv_store_port_attribute(xv_driver_t *this, char *name) {
-  xv_portattribute_t  *attr;
-
   xcb_intern_atom_cookie_t atom_cookie;
   xcb_intern_atom_reply_t *atom_reply;
 
   xcb_xv_get_port_attribute_cookie_t get_attribute_cookie;
   xcb_xv_get_port_attribute_reply_t *get_attribute_reply;
   
-  attr = (xv_portattribute_t *)malloc( sizeof(xv_portattribute_t) );
-  attr->name = strdup(name);
-  
   pthread_mutex_lock(&this->main_mutex);
-  atom_cookie = xcb_intern_atom(this->connection, 0, strlen(attr->name), attr->name);
+  atom_cookie = xcb_intern_atom(this->connection, 0, strlen(name), name);
   atom_reply = xcb_intern_atom_reply(this->connection, atom_cookie, NULL);
   get_attribute_cookie = xcb_xv_get_port_attribute(this->connection, this->xv_port, atom_reply->atom);
   get_attribute_reply = xcb_xv_get_port_attribute_reply(this->connection, get_attribute_cookie, NULL);
-  attr->value = get_attribute_reply->value;
   free(atom_reply);
-  free(get_attribute_reply);
   pthread_mutex_unlock(&this->main_mutex);
   
-  xine_list_push_back (this->port_attributes, attr);
+  if (get_attribute_reply != NULL) {
+    xv_portattribute_t *attr;
+
+    attr = (xv_portattribute_t *) malloc(sizeof(xv_portattribute_t));
+    attr->name = strdup(name);
+    attr->value = get_attribute_reply->value;
+    free(get_attribute_reply);
+
+    xine_list_push_back(this->port_attributes, attr);
+  }
 }
 
 static void xv_restore_port_attributes(xv_driver_t *this) {
@@ -1337,7 +1344,7 @@ static vo_driver_t *open_plugin(video_driver_class_t *class_gen, const void *vis
   else
     xprintf(class->xine, XINE_VERBOSITY_LOG,
 	    _("video_out_xcbxv: using Xv port %d from adaptor %s for hardware "
-	      "colorspace conversion and scaling.\n"), xv_port,
+	      "colour space conversion and scaling.\n"), xv_port,
             xcb_xv_adaptor_info_name(adaptor_it.data));
   
   this->xv_port           = xv_port;
@@ -1409,7 +1416,7 @@ static vo_driver_t *open_plugin(video_driver_class_t *class_gen, const void *vis
 	
 	if(!strcmp(xcb_xv_attribute_info_name(attribute_it.data), "XV_HUE")) {
 	  if (!strncmp(xcb_xv_adaptor_info_name(adaptor_it.data), "NV", 2)) {
-            xprintf (this->xine, XINE_VERBOSITY_NONE, "video_out_xcbxv: ignoring broken XV_HUE settings on NVidia cards");
+            xprintf (this->xine, XINE_VERBOSITY_NONE, "video_out_xcbxv: ignoring broken XV_HUE settings on NVidia cards\n");
 	  } else {
 	    xv_check_capability (this, VO_PROP_HUE, attribute_it.data,
 			         adaptor_it.data->base_id,
@@ -1444,7 +1451,7 @@ static vo_driver_t *open_plugin(video_driver_class_t *class_gen, const void *vis
 			       adaptor_it.data->base_id,
 			       "video.device.xv_autopaint_colorkey",
 			       _("autopaint colour key"),
-			       _("Make Xv autopaint its colorkey."));
+			       _("Make Xv autopaint its colour key."));
 
 	} else if(!strcmp(xcb_xv_attribute_info_name(attribute_it.data), "XV_FILTER")) {
 	  int xv_filter;
