@@ -15,10 +15,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: utils.c,v 1.42 2005/09/05 17:02:57 valtri Exp $
- *
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  */
 #define	_POSIX_PTHREAD_SEMANTICS 1	/* for 5-arg getpwuid_r on solaris */
 
@@ -50,7 +47,7 @@
 #include <ucontext.h>
 #endif
 
-#ifdef HAVE_LANGINFO_CODESET
+#ifdef HAVE_NL_LANGINFO
 #include <langinfo.h>
 #endif
 
@@ -70,7 +67,7 @@ typedef struct {
 /*
  * information about locales used in xine
  */
-static lang_locale_t lang_locales[] = {
+static const lang_locale_t lang_locales[] = {
   { "af_ZA",    "iso-8859-1",  "iso-8859-1",  NULL       },
   { "ar_AE",    "iso-8859-6",  "iso-8859-6",  NULL       },
   { "ar_BH",    "iso-8859-6",  "iso-8859-6",  NULL       },
@@ -434,6 +431,7 @@ char *xine_chomp(char *str) {
  * a thread-safe usecond sleep
  */
 void xine_usec_sleep(unsigned usec) {
+#if 0
 #if HAVE_NANOSLEEP
   /* nanosleep is prefered on solaris, because it's mt-safe */
   struct timespec ts, remaining;
@@ -449,6 +447,15 @@ void xine_usec_sleep(unsigned usec) {
   usleep(usec);
 #  endif
 #endif
+#else
+  if (usec < 10000) {
+      usec = 10000;
+  }
+  struct timeval tm;
+  tm.tv_sec  = usec / 1000000;
+  tm.tv_usec = usec % 1000000;
+  select(0, 0, 0, 0, &tm);
+#endif
 }
 
 
@@ -488,15 +495,24 @@ void xine_hexdump (const char *buf, int length) {
 }
 
 
-static const lang_locale_t *_get_first_lang_locale(char *lcal) {
+static const lang_locale_t *_get_first_lang_locale(const char *lcal) {
   const lang_locale_t *llocale;
+  int lang_len;
+  char *mod;
 
-  if(lcal && strlen(lcal)) {
+  if(lcal && *lcal) {
     llocale = &*lang_locales;
+
+    if ((mod = strchr(lcal, '@')))
+      lang_len = mod++ - lcal;
+    else
+      lang_len = strlen(lcal);
     
     while(llocale->language) {
-      if(!strncmp(lcal, llocale->language, strlen(lcal)))
-	return llocale;
+      if(!strncmp(lcal, llocale->language, lang_len)) {
+        if ((!mod && !llocale->modifier) || (mod && llocale->modifier && !strcmp(mod, llocale->modifier)))
+	  return llocale;
+      }
       
       llocale++;
     }
@@ -522,7 +538,8 @@ static char *_get_lang(void) {
 char *xine_get_system_encoding(void) {
   char *codeset = NULL;
   
-#ifdef HAVE_LANGINFO_CODESET
+#ifdef HAVE_NL_LANGINFO
+  setlocale(LC_ALL, "");
   codeset = nl_langinfo(CODESET);
 #endif
   /*
